/*
 * Copyright 2013-2014 Andrea "andynaz" Agnesse
 *
 * This file is part of "andyLib".
 *
 * "andyLib" is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * "andyLib" is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * "andyLib".  If not, see <http://www.gnu.org/licenses/>.
 *
 * author e-mail: andynaz@gmail.com
 * author home page: http://andynaz.altervista.org
 * program official page: http://andynaz.altervista.org/Progetti/andyLib.shtm
 */

package it.andynaz.utils;

import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;

/**
 * Useful method to manage a program that uses the andyLib project.
 *
 * @author andynaz
 * @version 2013/12/18
 */
public class UtilsProgram {
	// hides the constructor
	private UtilsProgram(){}
	
	/**
	 * Print a program header.
	 * 
	 * The header is read from a {@code header.txt} file in the package {@code
	 * it/andynaz/resources} (it has to be stored somewhere in the classpath).
	 * 
	 * <p>To be printed "well" on most terminal, each line should have a maximum
	 * length of 80 characters.</p>
	 */
	public static void printHeader() {
		InputStream in = Object.class.getResourceAsStream("/it/andynaz/resources/header.txt");
		if (in==null) return;
		UtilsIO.print(in);
	}
	
	
	/**
	 * Print a program help.
	 * 
	 * The header is read from a {@code help.txt} file in the package {@code
	 * it/andynaz/resources} (it has to be stored somewhere in the classpath).
	 * 
	 * <p>To be printed "well" on most terminal, each line should have a maximum
	 * length of 80 characters.</p>
	 */
	public static void printHelp() throws Exception{
		InputStream in = Object.class.getResourceAsStream("/it/andynaz/resources/help.txt");
		if (in==null) return;
		UtilsIO.print(in);
	}
	
	/**
	 * Keeps informations about the program versions.
	 */
	private static Properties versionData = new Properties();
	
	/**
	 * Reads the version data from the file.
	 * 
	 * The version data are loaded in an inner properties object; the loading is
	 * done only once on the first call of either {@link #getVersion()
	 * getVersion} or {@link #getBuildDate() getBuildDate}.
	 * 
	 * @throws IOException if the version data could not be read
	 */
	private static void loadVersionData() throws IOException{
		if (versionData.isEmpty()) {
			InputStream in = Object.class.getResourceAsStream("/it/andynaz/resources/version.properties");
			if (in==null) return;
			versionData.load(in);
		}
	//	versionData.list(System.out);
	}
	
	/**
	 * Returns the version of the project.
	 * 
	 * @return the version of the project
	 * 
	 * @throws IOException if the version data could not be read
	 */
	public static String getVersion() throws IOException{
		loadVersionData();
		return versionData!=null ? versionData.getProperty("version") : null;
	}
	
	/**
	 * Returns the build date of the project.
	 * 
	 * @return the build date of the project
	 * 
	 * @throws IOException if the version data could not be read
	 */
	public static String getBuildDate() throws IOException{
		loadVersionData();
		return versionData!=null ? versionData.getProperty("date") : null;
	}
	
}
